class OrderError(Exception):
    pass
class DimensionError(Exception):
    pass
class IncompatibleOrder(Exception):
    pass
class InvalidOperation(Exception):
    pass
class OperationNotAllow(Exception):
     pass
class OutOfRange(Exception):
    pass
